/*
 * SPDX-License-Identifier: AGPL-3.0-only
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published
 * by the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>
 */

const fs = require("fs");
const path = require("path");
const { execFileSync } = require("child_process");

const pkg = require("../../package.json");

function getVersion() {
  if (process.env.TAG) {
    return process.env.TAG;
  }

  return pkg.version;
}

const version = getVersion();

const goVersionFile = path.join(__dirname, "../..", "config/version.go");

function main() {
  writeVersionFile();
  execFileSync("git", ["add", goVersionFile]);
}

function writeVersionFile() {
  fs.writeFileSync(
    goVersionFile,
    [
      "// Code generated by scripts/bump.js. DO NOT EDIT.",
      "",
      "package config",
      "",
      `const Version = "${version}"`,
      "",
    ].join("\n")
  );
}

main();
